@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 """Django page CMS ``managers``."""
 from pages import settings
+from pages.cache import cache
 from pages.utils import normalize_url
 from pages.http import get_slug
 
 from django.db import models, connection
 from django.db.models import Q
-from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist
 from django.contrib.auth.models import User, SiteProfileNotAvailable
 from django.db.models import Avg, Max, Min, Count
@@ -20,6 +20,10 @@ from datetime import datetime
 
 ISODATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f' # for parsing dates from JSON
 
+# This affects how content is saved
+CATEGORY_INSTALLED = 'pages.plugins.category' in global_settings.INSTALLED_APPS
+CATEGORY_TYPE = 'category'
+
 
 class PageManager(TreeManager):
     """
@@ -325,6 +329,12 @@ class ContentManager(models.Manager):
             for c in oldest_content:
                 c.delete()
 
+        # Delete also old category if relevant
+        if CATEGORY_INSTALLED and ctype == CATEGORY_TYPE:
+            cat = self.filter(page=page, language=language, type=ctype)
+            cat = cat.exclude(pk=content.pk)
+            cat.delete()
+
         return content
 
     def get_content_object(self, page, language, ctype):
@@ -354,6 +364,9 @@ class ContentManager(models.Manager):
         frozen = int(bool(page.freeze_date))
         key = self.PAGE_CONTENT_DICT_KEY % (page.id, ctype, frozen)
 
+        # Spaces do not work with memcache
+        key = key.replace(' ', '-')
+
         if page._content_dict is None:
             page._content_dict = dict()
         if page._content_dict.get(key, None):
