@@ -1,4 +1,5 @@
 from django.test import TestCase
+from pages.cache import cache
 from pages.models import Page, Content
 from pages import settings as pages_settings
 from pages.testproj import test_settings
@@ -42,6 +43,7 @@ class TestCase(TestCase):
         self.old_url_conf = getattr(settings, 'ROOT_URLCONF')
         setattr(settings, 'ROOT_URLCONF', 'pages.testproj.urls')
         clear_url_caches()
+        cache.clear()
 
     def tearDown(self):
         setattr(settings, 'ROOT_URLCONF', self.old_url_conf)
@@ -82,7 +84,10 @@ class TestCase(TestCase):
     def reset_urlconf(self):
         url_conf = getattr(settings, 'ROOT_URLCONF', False)
         if url_conf:
-            reload(import_module(url_conf))
+            try:
+                reload(import_module(url_conf))
+            except:
+                pass
         reload(import_module('pages.urls'))
         reload(import_module('pages.testproj.urls'))
         clear_url_caches()
@@ -100,7 +105,7 @@ class TestCase(TestCase):
         return page_data
 
 
-    def new_page(self, content={'title':'test-page'}, language='en-us'):
+    def new_page(self, content={'title': 'test-page'}, language='en-us'):
         author = User.objects.all()[0]
         page = Page.objects.create(author=author, status=Page.PUBLISHED,
             template='pages/examples/index.html')
@@ -121,4 +126,5 @@ class TestCase(TestCase):
         self.assertRedirects(response, '/admin/pages/page/')
         slug_content = Content.objects.get_content_slug_by_slug(
             page_data['slug'])
-        return slug_content.page
\ No newline at end of file
+        return slug_content.page
+
